
# include <iostream>
# include "Env.hh"  

# include <string>
using namespace std;

// JOB is a global variable currently needed by the C code. Will be
// removed soon.
Job* JOB = NULL;

namespace SomeName {
  // ******************************************************************
  // CONSTRUCTORS

  Env::Env() {
    // For now, we are relying on the C Job structure.
    if ( JOB == NULL ) {
      Initialize();
    }
    Job * job = JOB;
    job_init(job);
    this->_job = job;
  }

  void Env::Initialize() {
    JOB = new Job;
    job_init(JOB);
  }

  // ******************************************************************
  // ACCESSORS
  

  // ******************************************************************
  // METHODS

  /// Add a new Page for file 'file'.
  ///
  /// Creates a Page instances, makes it the current page within
  /// this Env, adds it to the 'pages' vector, and returns it.
  Page* Env::registerPage(const string& file) {
    Page* page = new Page(this, file);
    this->currentIs(page);
    this->pagesRef().push_back(page);
    return page;
  }

  /// Initialize various fields based on command-line flags.
  bool Env::processArgs(int argc, char** argv) {
    // NOTE: We cannot use the 'process_args' from the C
    // implementation because it isn't part of the lib (it
    // is in 'gocr.c' which defines the 'main' method too).

    int i;
    char *s1;
    Job* job = this->jobRef();
    assert(job);
    if (argc <= 1) {
      //out_version(1);
      exit(0);
    }
#ifdef HAVE_PGM_H
    pnm_init(&argc, &argv);
#endif

    bool result = false;
    job->src.fname = NULL;
    /* process arguments */
    for (i = 1; i < argc; i++) {
      if (strcmp(argv[i], "--help") == 0) {
        result = false;
        goto EndArgs;
      }
      if (argv[i][0] == '-' && argv[i][1] != 0) {
        s1 = "";
        if (i + 1 < argc)
          s1 = argv[i + 1];
        switch (argv[i][1]) {
          case 'h': /* help */
            result = false;
            goto EndArgs;
            break;
          case 'i': /* input image file */
            job->src.fname = s1;
            i++;
            break;
          case 'e': /* logging file */
            if (s1[0] == '-' && s1[1] == '\0') {
#ifdef HAVE_UNISTD_H
              dup2(STDOUT_FILENO, STDERR_FILENO); /* -e /dev/stdout  works */
#else
              fprintf(stderr, "stderr redirection not possible without unistd.h\n");
#endif           
            }
            else if (!freopen(s1, "w", stderr)) {
              fprintf(stderr, "stderr redirection to %s failed\n", s1);
            }
            i++;
            break;
          case 'p': /* database path */
            job->cfg.db_path=s1;
            i++;
            break;
          case 'o': /* output file */
            if (s1[0] == '-' && s1[1] == '\0') {        /* default */
            }
            else if (!freopen(s1, "w", stdout)) {
              fprintf(stderr, "stdout redirection to %s failed\n", s1);
            };
            i++;
            break;
          case 'f': /* output format */
            if (strcmp(s1, "ISO8859_1") == 0) job->cfg.out_format=ISO8859_1; else
              if (strcmp(s1, "TeX")       == 0) job->cfg.out_format=TeX; else
                if (strcmp(s1, "HTML")      == 0) job->cfg.out_format=HTML; else
                  if (strcmp(s1, "XML")       == 0) job->cfg.out_format=XML; else
                    if (strcmp(s1, "SGML")      == 0) job->cfg.out_format=SGML; else
                      if (strcmp(s1, "UTF8")      == 0) job->cfg.out_format=UTF8; else
                        if (strcmp(s1, "ASCII")     == 0) job->cfg.out_format=ASCII; else
                          fprintf(stderr,"Warning: unknown format (-f %s)\n",s1);
            i++;
            break;
          case 'c': /* list of chars (_ = not recognized chars) */
            job->cfg.lc = s1;
            i++;
            break;
          case 'C': /* char filter, default: NULL (all chars) */
            /* ToDo: UTF8 input, wchar */
            job->cfg.cfilter = s1;
            i++;
            break;
          case 'd': /* dust size */
            job->cfg.dust_size = atoi(s1);
            i++;
            break;
          case 'l': /* grey level 0<160<=255, 0 for autodetect */
            job->cfg.cs = atoi(s1);
            i++;
            break;
          case 's': /* spacewidth/dots (0 = autodetect) */
            job->cfg.spc = atoi(s1);
            i++;
            break;
          case 'v': /* verbose mode */
            job->cfg.verbose |= atoi(s1);
            i++;
            break;
          case 'm': /* operation modes */
            job->cfg.mode |= atoi(s1);
            i++;
            break;
          case 'n': /* numbers only */
            job->cfg.only_numbers = atoi(s1);
            i++;
            break;
          case 'x': /* initialize progress output s1=fname */
            cerr << "ERORR: -x currently disabled in C++ implementation" << endl;
            // ini_progress(s1);
            i++;
            break;
          case 'a': /* set certainty */
            job->cfg.certainty = atoi(s1);;
            i++;
            break;
          default:
            fprintf(stderr, "# unknown option use -h for help\n");
        }
        continue;
      }
      else /* argument can be filename v0.2.5 */ if (argv[i][0] != '-'
                                                     || argv[i][1] == '\0' ) {
        job->src.fname = argv[i];
      }
    }

    EndArgs:
    if ( job->src.fname ) {
      // If we've established an input file, we automatically create a
      // Page for it.
      this->registerPage(job->src.fname);
    }
    return result;
  }
  
  void Env::wrapup() {
    // cerr << "NOTE: Finished" << endl;

  }

  int Env::verbose() const {
    return this->job()->cfg.verbose;
  }

  void Env::test() {
    cerr << "ERROR: Not unit test for Env - FIX THIS!" << endl;
  }

};
